package com.inet.authentication.passwordfile;

import java.util.HashMap;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.servlet.http.HttpServletRequest;

import com.inet.authentication.AuthenticationDescription;
import com.inet.authentication.AuthenticationDescription.BasicSupport;
import com.inet.authentication.AuthenticationProvider;
import com.inet.authentication.LoginProcessor;
import com.inet.authentication.passwordfile.structure.PasswordFileAuthStructureProvider;
import com.inet.config.Configuration;
import com.inet.http.servlet.ClientLocale;

/**
 * AuthenticationProvider for password file.
 */
public class PasswordFileAuthProvider implements AuthenticationProvider {

    // the login type saved in the configuration, every AuthenticationProvider must return an unique name
    public static final String NAME         = "passwordfile";

    // cache for the loaded password file
    private final PasswordFile passwordFile = new PasswordFile();

    /**
     * {@inheritDoc}
     */
    @Override
    public String name() {
        return NAME;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int getPriority() {
        return 444;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public LoginProcessor create( AuthenticationDescription description ) {
        String path = description.getSettings().get( PasswordFileAuthStructureProvider.PASSWORD_FILE );
        passwordFile.setPasswordFile( path );
        return new PasswordFileAuthWUI( description, passwordFile );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getDisplayName( Map<String, String> settings ) {
        try {
            ResourceBundle bundle = ResourceBundle.getBundle( "com.inet.authentication.passwordfile.structure.i18n.ConfigStructure", ClientLocale.getThreadLocale() );
            return bundle.getString( name() );
        } catch( MissingResourceException ex ) {
            return name();
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public AuthenticationDescription getAuthenticationDescription( Map<String, String> settings, boolean isApplication, boolean isProxy ) {
        return new AuthenticationDescription( this, settings, name(), BasicSupport.No );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public LoginProcessor createLoginProcessor( AuthenticationDescription description, String authorization, HttpServletRequest request, boolean isApplication ) {
        if( authorization == null || !authorization.startsWith( "Digest " ) ) {
            return null;
        }
        return create( description );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Map<String, String> applySettings( Map<String, String> clientProperties, Configuration config ) {
        HashMap<String, String> result = new HashMap<>();
        result.put( PasswordFileAuthStructureProvider.PASSWORD_FILE, clientProperties.get( PasswordFileAuthStructureProvider.PASSWORD_FILE ) );
        return result;
    }
}
